/*
 * Copyright (c) 2006-2007 The tyndur Project. All rights reserved.
 *
 * This code is derived from software contributed to the tyndur Project
 * by Burkhard Weseloh.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdio.h"

#include "jprintf.h"

struct fprintf_args
{
    FILE * fp;
};

int fprintf_putc(struct fprintf_args * arg, char c)
{
    fputc(c, arg->fp);

    return 1;
}

int fprintf_putsn(struct fprintf_args * arg, const char * string, int n)
{
    return fwrite(string, 1, (n == -1) ? strlen(string) : n, arg->fp);
}

int vfprintf(FILE * fp, const char * format, va_list ap)
{
    struct fprintf_args args = { fp };
    struct jprintf_args fprintf_handler = { (pfn_putc)&fprintf_putc, 0, (void*)&args };

	return jvprintf(&fprintf_handler, format, ap);
}

int fprintf(FILE * fp, const char * format, ...)
{
	va_list ap;
	int retval;

	va_start(ap, format);
	retval = vfprintf(fp, format, ap);
	va_end(ap);

	return retval;
}
